import pytest
from downerhelper.qgis import *
from downerhelper.logs import setup_queue
import os
from uuid import uuid4

# timestamp = 1728342288000
# timestamp_date = '2024-10-08T12:04:48+13:00'

# @pytest.fixture(scope="module")
# def qgis_creds():
#     root = os.getenv('QGIS_ROOT')
#     username = os.getenv('QGIS_USERNAME')
#     pwd = os.getenv('QGIS_PWD')
#     return root, username, pwd

@pytest.fixture(scope="module")
def queue(keyvault_url):
    return setup_queue('pytest', str(uuid4()), os.getenv('LOG_TABLE'), 'db-config-1', keyvault_url)

@pytest.fixture(scope="module")
def token(qgis_creds, queue):
    root, username, pwd = qgis_creds
    return get_access_token(root, username, pwd, queue)

def test_get_token(qgis_creds, queue):
    root, username, pwd = qgis_creds    
    token = get_access_token(root, username, pwd, queue)
    assert token != None
    assert isinstance(token, str)
    assert len(token) > 0

    with pytest.raises(Exception):
        get_access_token('https://gis.nosite.co.au', username, pwd, queue)

    with pytest.raises(Exception):
        get_access_token(root, 'username', pwd, queue)

    with pytest.raises(Exception):
        get_access_token(root, username, 'password', queue)

def test_query_feature_service(token, queue):
    query_result = query_feature_service(token, os.getenv('FEATURE_SERVICE'), '1=1', queue)
    assert isinstance(query_result, dict)

    expected = [
        {
            "marktest": None,
            "test_string": None
        },
        {
            "marktest": 'mark1',
            "test_string": 'string1'
        },
        {
            "marktest": 'mark2',
            "test_string": 'test string 2'
        }
    ]
    found = 0
    for feature in query_result['features']:
       attributes = feature['attributes']
       for expect in expected:
            field1 = attributes['marktest'] == expect['marktest']
            field2 = attributes['test_string'] == expect['test_string']
            if not (field1 and field2): continue
            found += 1
            break
    assert found == len(expected)

    with pytest.raises(Exception):
        query_feature_service(token, os.getenv('FEATURE_SERVICE'), '1', queue)

    with pytest.raises(Exception):
        query_feature_service('', os.getenv('FEATURE_SERVICE'), '1=1', queue)

def test_query_get_attachments(token, queue):
    object_id = '3'
    attachment_data = query_attachments(token, os.getenv('FEATURE_SERVICE'), object_id, queue)
    assert attachment_data['attachmentGroups'] == []

    object_id = '2'
    attachment_data = query_attachments(token, os.getenv('FEATURE_SERVICE'), object_id, queue)
    expected = [
        {
            'att_name': 'test.txt',
            'content_type': 'text/plain',
            'name': 'test.txt',
        },
        {
            'att_name': 'B3018_discsLabels.jpg',
            'content_type': 'image/jpeg',
            'name': 'B3018_discsLabels.jpg',
        }
    ]
    found = 0
    for group in attachment_data['attachmentGroups']:
        for info in group['attachmentInfos']:
            attachment = get_attachment(token, os.getenv('FEATURE_SERVICE'), object_id, info['attachmentid'], queue)
            assert isinstance(attachment, bytes)
            for expect in expected:
                field1 = info['name'] == expect['name']
                field2 = info['att_name'] == expect['att_name']
                field3 = info['contentType'] == expect['content_type']
                if not (field1 and field2 and field3): continue
                found += 1
                break
            
    assert found == len(expected)

    with pytest.raises(Exception):
        query_attachments(token, os.getenv('FEATURE_SERVICE'), 'none_id', queue)

def test_update_feature_service(token, queue):
    features = [
        {
            'attributes': {
                'marktest': str(uuid4()),
                'test_string': str(uuid4()),
                'objectid': 4
            }
        }
    ]
    assert update_feature_service(token, os.getenv('FEATURE_SERVICE'), features, queue)

    query_result = query_feature_service(token, os.getenv('FEATURE_SERVICE'), '1=1', queue)
    for feature in query_result['features']:
        attributes = feature['attributes']
        if attributes['objectid'] != features[0]['attributes']['objectid']: continue
        assert attributes['marktest'] == features[0]['attributes']['marktest']
        assert attributes['test_string'] == features[0]['attributes']['test_string']

    features = [
        {
            'attributes': {
                'marktest': str(uuid4()),
                'test_string': str(uuid4()),
                'objectid': -1
            }
        }
    ]
    with pytest.raises(Exception):
        update_feature_service(token, os.getenv('FEATURE_SERVICE'), features, queue)

    with pytest.raises(Exception):
        update_feature_service(token, os.getenv('FEATURE_SERVICE'), None, queue)

def test_timestamp_to_datetime(queue):
    date = timestamp_to_datetime(os.getenv('TIMESTAMP'), queue)
    assert date == os.getenv('TIMESTAMP_TO_DATETIME')


import logging
import psycopg2 as pg
from datetime import datetime, timezone
from downerhelper.secrets import get_config_dict

class PostgresLogQueue():
    def __init__(self, logger_name, job_id, table, db_config):
        try:
            if '' in [logger_name, job_id, table] or \
                None in [logger_name, job_id, table] or db_config == {}:
                raise Exception("Invalid parameters")
            self.logger_name = logger_name
            self.job_id = job_id
            self.db_config = db_config
            self.table = table
            self.queue = [
                {
                    'levelname': 'INFO',
                    'message': f'queue: {logger_name} created for job_id: {job_id}',
                    'created_at': datetime.now(timezone.utc)
                }
            ]
        except Exception as e:
            logging.error(f"Error setting up PostgresLogHandler: {e}")
            raise e

    def add(self, levelname, message):
        self.queue.append({
            'levelname': levelname,
            'message': message,
            'created_at': datetime.now(timezone.utc)
        })

    def save(self):
        try:
            conn = pg.connect(**self.db_config)
            cursor = conn.cursor()
            cursor.execute("set time zone 'UTC'")
            cursor.execute(f"""
            create table if not exists {self.table} (
                id serial primary key,
                created_at timestamptz default now(),
                name varchar(255),
                levelname varchar(50),
                message text,
                job_id varchar(255) not null
            )""")
            conn.commit()
            
            for log in self.queue:
                cursor.execute(f"""
                insert into {self.table} (name, levelname, message, job_id, created_at)
                values (%s, %s, %s, %s, %s)
                """, (self.logger_name, log['levelname'], log['message'], self.job_id, log['created_at']))
            conn.commit()

            if cursor: cursor.close()
            if conn: conn.close()

            self.queue = []

        except Exception as e:
            logging.error(f"Error saving logs: {e}")
            if cursor: cursor.close()
            if conn: conn.close()

def setup_queue(logger_name, job_id, table, db_config_name, keyvault_url):
    try:
        db_config = get_config_dict(db_config_name, keyvault_url)
        return PostgresLogQueue(logger_name, job_id, table, db_config)
    except Exception as e:
        logging.error(f"Error setting up logger: {e}")
        raise Exception("Error setting up logger")


import pytest
from downerhelper.secrets import get_secret_value, get_config_dict, get_key_url
from downerhelper.logs import setup_queue
import os 
import logging 
from uuid import uuid4

@pytest.fixture(scope="module")
def queue():
    return setup_queue('pytest', str(uuid4()), os.getenv('LOG_TABLE'), 'db-config-1', os.getenv('KEYVAULT_URL'))

def test_valid_secret_logging(queue):
    value = get_secret_value('test-secret', os.getenv('KEYVAULT_URL'))
    assert value == 'dfaec67d-cf0a-44e8-b5df-6e8346092450'

def test_valid_secret_logging(queue):
    value = get_secret_value('test-secret', os.getenv('KEYVAULT_URL'), queue)
    assert value == 'dfaec67d-cf0a-44e8-b5df-6e8346092450'

def test_invalid_secret(queue):
    value = get_secret_value('invalid', os.getenv('KEYVAULT_URL'), queue)
    assert value == None

def test_invalid_keyvault(queue):
    value = get_secret_value('test-secret', 'https://invalid.vault.azure.net', queue)
    assert value == None

def test_get_config_dict(queue):
    secret_name = 'db-config-test'
    db_config = get_config_dict(secret_name, os.getenv('KEYVAULT_URL'))
    assert db_config == {
        'dbname': 'value0',
        'user': 'value1',
        'password': 'VALue2',
        'host': 'value3'
    }

    with pytest.raises(Exception):
        get_config_dict('invalid', os.getenv('KEYVAULT_URL'))

def test_get_key_url(queue):
    key, url = get_key_url('test-key-url', os.getenv('KEYVAULT_URL'), queue)
    assert key == 'aa'
    assert url == 'bb/cc/dd'

    with pytest.raises(Exception):
        get_key_url('invalid', os.getenv('KEYVAULT_URL'), queue)